# -*- coding: utf-8 -*-
"""Project metadata

Information describing the project.
"""

# The package name, which is also the "UNIX name" for the project.
package = 'nagios_graphite'
project = "Nagios Graphite"
project_no_spaces = project.replace(' ', '')
version = '0.1.3'
description = 'Nagios command for Graphite metrics'
authors = ['Michael-Keith Bernard']
authors_string = ', '.join(authors)
emails = ['mkbernard.dev@gmail.com']
license = 'MIT'
copyright = '2015 ' + authors_string
url = 'https://github.com/segfaultax/nagios_graphite'



# -*- coding: utf-8 -*-
"""Nagios command for Graphite metrics"""

from nagios_graphite import metadata


__version__ = metadata.version
__author__ = metadata.authors[0]
__license__ = metadata.license
__copyright__ = metadata.copyright



# -*- coding: utf-8 -*-

# The parametrize function is generated, so this doesn't work:
#
#     from pytest.mark import parametrize
#

import re
import json
import shlex
import random
import urllib

import pytest
import responses
parametrize = pytest.mark.parametrize

# from nagios_graphite import metadata
from nagios_graphite import main
from nagios_graphite.main import FUNCTIONS, combine, GraphiteNagios

xs = range(1000)
random.shuffle(xs)


def test_sum():
    assert FUNCTIONS["sum"](xs) == sum(xs)


def test_min():
    assert FUNCTIONS["min"](xs) == min(xs)


def test_max():
    assert FUNCTIONS["max"](xs) == max(xs)


def test_avg():
    assert FUNCTIONS["avg"](xs) == sum(xs) / len(xs)


def test_median():
    assert FUNCTIONS["median"](xs) == 500


def test_95th():
    assert FUNCTIONS["95th"](xs) == 950


def test_99th():
    assert FUNCTIONS["99th"](xs) == 990


def test_999th():
    assert FUNCTIONS["999th"](xs) == 999


graphite_without_none = [
    {"target": "foo", "datapoints": [[1, 10], [2, 11], [3, 12]]},
    {"target": "bar", "datapoints": [[4, 10], [5, 11], [6, 12]]},
]
graphite_with_none = [
    {"target": "foo", "datapoints": [[1, 10], [2, 11], [3, 12]]},
    {"target": "bar", "datapoints": [[None, 10], [5, 11], [None, 12]]},
]


def test_combine_empty():
    assert combine([], sum) == 0


def test_combine_without_none():
    assert combine(graphite_without_none, sum) == 21


def test_combine_with_none():
    assert combine(graphite_with_none, sum) == 11


def options_for(s):
    argv = shlex.split("nagios_graphite " + s)
    return GraphiteNagios(argv).options


def test_format_from():
    assert main.format_from("1second") == "-1second"
    assert main.format_from("-1second") == "-1second"


def test_graphite_querystring():
    opts = options_for("-M 'cpu.load.average' -F 1minute")
    qs = main.graphite_querystring(opts)

    assert urllib.urlencode({"from": main.format_from("1minute")}) in qs
    assert urllib.urlencode({"target": "cpu.load.average"}) in qs
    assert urllib.urlencode({"format": "json"}) in qs


def test_graphite_url():
    opts = options_for("-M 'cpu.load.average' -H http://example.com")
    expected = "{}?{}".format(
        "http://example.com", main.graphite_querystring(opts))

    assert main.graphite_url(opts) == expected


def test_graphite_session_noauth():
    opts = options_for("-M 'cpu.load.average' -H http://example.com")
    assert main.graphite_session(opts).auth is None


def test_graphite_session_auth():
    opts = options_for(
        "-M 'cpu.load.average' -H http://example.com -U foo -P pass")
    assert main.graphite_session(opts).auth == ("foo", "pass")


@responses.activate
def test_graphite_fetch_success():
    opts = options_for(
        "-M 'cpu.load.average' -H http://example.com -U foo -P pass")
    url_re = re.compile("^{}.*$".format(re.escape(opts.hostname)))

    resp = json.dumps(graphite_with_none)
    responses.add(
        responses.GET, url_re,
        body=resp, status=200,
        content_type='application/json')

    assert main.graphite_fetch(opts) == graphite_with_none


@responses.activate
def test_graphite_fetch_failure():
    opts = options_for(
        "-M 'cpu.load.average' -H http://example.com -U foo -P pass")
    url_re = re.compile("^{}.*$".format(re.escape(opts.hostname)))

    responses.add(
        responses.GET, url_re, status=500)

    assert main.graphite_fetch(opts) == []


@responses.activate
def test_check_graphite_success():
    for fn_name, aggfn in FUNCTIONS.iteritems():
        opts = options_for(
            "-M 'cpu.load.average' -H http://example.com -U foo -P pass "
            "-A {}".format(fn_name))
        url_re = re.compile("^{}.*$".format(re.escape(opts.hostname)))

        resp = json.dumps(graphite_with_none)
        responses.add(
            responses.GET, url_re,
            body=resp, status=200,
            content_type='application/json')

        expected = main.combine(graphite_with_none, aggfn)
        assert main.check_graphite(opts) == expected


@responses.activate
def test_check_graphite_failure():
    for fn_name, aggfn in FUNCTIONS.iteritems():
        opts = options_for(
            "-M 'cpu.load.average' -H http://example.com -U foo -P pass "
            "-A {}".format(fn_name))
        url_re = re.compile("^{}.*$".format(re.escape(opts.hostname)))

        responses.add(
            responses.GET, url_re, status=500)

        assert main.check_graphite(opts) is None



#!/usr/bin/env python
# -*- coding: utf-8 -*-
"""Program entry point"""

from __future__ import print_function

import sys
import urllib

import requests
from pynagios import Plugin, Response, make_option, UNKNOWN


def percentile(n):
    """Create function that calculates percentile for list"""

    return lambda xs: sorted(xs)[int(len(xs)*n)]

FUNCTIONS = {
    "sum": sum,
    "min": min,
    "max": max,
    "avg": lambda xs: sum(xs) / len(xs),
    "median": percentile(0.5),
    "95th":   percentile(0.95),
    "99th":   percentile(0.99),
    "999th":  percentile(0.999)
}
F_OPTS = ", ".join(FUNCTIONS.keys())


def combine(series, aggfn):
    """Combine Graphite series data using aggfn"""

    pairs = reduce(lambda a, b: a + b, [e["datapoints"] for e in series], [])
    points = [e[0] for e in pairs if e[0] is not None]
    return aggfn(points)


def format_from(from_):
    if not from_.startswith("-"):
        return "-" + from_
    else:
        return from_


def graphite_querystring(opts):
    qs = {
        "target": opts.target,
        "from": format_from(opts.from_),
        "format": "json",
    }

    return urllib.urlencode(qs)


def graphite_url(opts):
    qs = graphite_querystring(opts)
    return "{}?{}".format(opts.hostname, qs)


def graphite_session(opts):
    session = requests.Session()
    if opts.username:
        session.auth = (opts.username, opts.password)
    return session


def graphite_fetch(opts, session=None):
    if session is None:
        session = graphite_session(opts)

    url = graphite_url(opts)
    resp = session.get(url)

    return resp.json() if resp.ok else []


def check_graphite(opts, session=None):
    aggfn = FUNCTIONS[opts.func]
    raw_data = graphite_fetch(opts)

    if raw_data:
        return combine(raw_data, aggfn)
    else:
        return None


class GraphiteNagios(Plugin):
    username = make_option(
        "--username", "-U",
        help="Username (HTTP Basic Auth)")
    password = make_option(
        "--password", "-P",
        help="Password (HTTP Basic Auth)")

    name = make_option(
        "--name", "-N",
        help="Metric name", default="metric")
    target = make_option(
        "--target", "-M",
        help="Graphite target (series or query)")
    from_ = make_option(
        "--from", "-F",
        help="Starting offset", default="1minute")

    func = make_option(
        "--algorithm", "-A",
        help=("Algorithm for combining metrics, options: "
              "{}, (default: avg)".format(F_OPTS)),
        default="avg", choices=FUNCTIONS.keys())

    def check(self):
        value = check_graphite(self.options)
        if value is None:
            return Response(UNKNOWN, "No results returned!")

        message = "{} ({} is {})".format(
            self.options.name, self.options.func, value)
        response = self.response_for_value(value, message)
        response.set_perf_data(self.options.func, value)
        return response


def main(args):
    try:
        return GraphiteNagios(args).check().exit()
    except Exception as e:
        message = "{}: {}".format(e.__class__.__name__, str(e))
        Response(UNKNOWN, "Client error: " + message).exit()


def entry_point():
    return main(sys.argv)

if __name__ == '__main__':
    entry_point()
